/**
 * @file iap.h
 * @brief CycloneBOOT IAP User API
 *
 * @section License
 *
 * Copyright (C) 2010-2022 Oryx Embedded SARL. All rights reserved.
 *
 * This file is part of CycloneBOOT Eval.
 *
 * This software is provided in source form for a short-term evaluation only. The
 * evaluation license expires 90 days after the date you first download the software.
 *
 * If you plan to use this software in a commercial product, you are required to
 * purchase a commercial license from Oryx Embedded SARL.
 *
 * After the 90-day evaluation period, you agree to either purchase a commercial
 * license or delete all copies of this software. If you wish to extend the
 * evaluation period, you must contact sales@oryx-embedded.com.
 *
 * This evaluation software is provided "as is" without warranty of any kind.
 * Technical support is available as an option during the evaluation period.
 *
 * @author Oryx Embedded SARL (www.oryx-embedded.com)
 * @version 2.0.0
 **/

#ifndef _IAP_H
#define _IAP_H

//Dependencies
#include <stdint.h>
#include "iap_config.h"
#include "drivers/flash/flash.h"
#include "core/crypto.h"
#include "hash/hash_algorithms.h"
#include "core/crc32.h"
#include "error.h"

//CycloneBOOT Version string
#define CYCLONE_BOOT_IAP_VERSION_STRING "1.0.0"
//CycloneBOOT Major version
#define CYCLONE_BOOT_IAP_MAJOR_VERSION 1
//CycloneBOOT Minor version
#define CYCLONE_BOOT_IAP_MINOR_VERSION 0
//CycloneBOOT Revision number
#define CYCLONE_BOOT_IAP_REV_NUMBER 0

//Integrity check support
#ifndef IAP_INTEGRITY_SUPPORT
   #define IAP_INTEGRITY_SUPPORT ENABLED
#elif (IAP_INTEGRITY_SUPPORT != ENABLED && IAP_INTEGRITY_SUPPORT != DISABLED)
   #error IAP_INTEGRITY_SUPPORT parameter is not valid
#endif

//Authentication check support
#ifndef IAP_AUTHENTICATION_SUPPORT
   #define IAP_AUTHENTICATION_SUPPORT DISABLED
#elif (IAP_AUTHENTICATION_SUPPORT != ENABLED && IAP_AUTHENTICATION_SUPPORT != DISABLED)
   #error IAP_AUTHENTICATION_SUPPORT parameter is not valid
#endif

//Signature check support
#ifndef IAP_SIGNATURE_SUPPORT
   #define IAP_SIGNATURE_SUPPORT DISABLED
#elif (IAP_SIGNATURE_SUPPORT != ENABLED && IAP_SIGNATURE_SUPPORT != DISABLED)
   #error IAP_SIGNATURE_SUPPORT parameter is not valid
#endif

//Acceptabled check support
#if !((IAP_INTEGRITY_SUPPORT == ENABLED && !(IAP_AUTHENTICATION_SUPPORT == ENABLED || \
   IAP_SIGNATURE_SUPPORT == ENABLED)) || \
   (IAP_AUTHENTICATION_SUPPORT == ENABLED && !(IAP_INTEGRITY_SUPPORT == ENABLED || \
   IAP_SIGNATURE_SUPPORT == ENABLED)) ||\
   (IAP_SIGNATURE_SUPPORT == ENABLED && !(IAP_AUTHENTICATION_SUPPORT == ENABLED || \
   IAP_INTEGRITY_SUPPORT == ENABLED)))
   #error Exactly one of the following parameters MUST be enabled : \
IAP_INTEGRITY_SUPPORT - \
IAP_AUTHENTICATION_SUPPORT - \
IAP_SIGNATURE_SUPPORT
#endif

//IAP single bank internal flash memory mode support
#ifndef IAP_SINGLE_BANK_SUPPORT
#define IAP_SINGLE_BANK_SUPPORT DISABLED
#elif (IAP_SINGLE_BANK_SUPPORT != ENABLED && IAP_SINGLE_BANK_SUPPORT != DISABLED)
   #error IAP_SINGLE_BANK_SUPPORT  parameter is not valid
#endif

//IAP dual bank internal flash memory mode support
#ifndef IAP_DUAL_BANK_SUPPORT
#define IAP_DUAL_BANK_SUPPORT ENABLED
#elif (IAP_DUAL_BANK_SUPPORT != ENABLED && IAP_DUAL_BANK_SUPPORT != DISABLED)
   #error IAP_DUAL_BANK_SUPPORT  parameter is not valid
#endif

//IAP fallback support
#ifndef IAP_FALLBACK_SUPPORT
#define IAP_FALLBACK_SUPPORT DISABLED
#elif (IAP_FALLBACK_SUPPORT != ENABLED && IAP_FALLBACK_SUPPORT != DISABLED)
   #error IAP_FALLBACK_SUPPORT  parameter is not valid
#endif

//Acceptabled internal memory mode
#if ((IAP_SINGLE_BANK_SUPPORT == ENABLED && IAP_DUAL_BANK_SUPPORT == ENABLED) || \
(IAP_SINGLE_BANK_SUPPORT == DISABLED && IAP_DUAL_BANK_SUPPORT == DISABLED))
   #error Exactly one of the following parameters MUST be enabled: \
IAP_SINGLE_BANK_SUPPORT - \
IAP_DUAL_BANK_SUPPORT
#endif

//IAP external memory encryption support
#ifndef IAP_EXT_MEM_ENCRYPTION_SUPPORT
#define IAP_EXT_MEM_ENCRYPTION_SUPPORT DISABLED
#elif (IAP_EXT_MEM_ENCRYPTION_SUPPORT != ENABLED && IAP_EXT_MEM_ENCRYPTION_SUPPORT != DISABLED)
   #error IAP_EXT_MEM_ENCRYPTION_SUPPORT  parameter is not valid
#endif

//Acceptable external memory encryption activation
#if ((IAP_EXT_MEM_ENCRYPTION_SUPPORT == ENABLED && IAP_SINGLE_BANK_SUPPORT == DISABLED))
   #error Encryption of external memory is available only in Signel Bank mode
#endif

//Cipher support
#ifndef IAP_ENCRYPTION_SUPPORT
   #define IAP_ENCRYPTION_SUPPORT DISABLED
#elif (IAP_ENCRYPTION_SUPPORT != ENABLED && IAP_ENCRYPTION_SUPPORT != DISABLED)
   #error IAP_ENCRYPTION_SUPPORT parameter is not valid
#endif

//RSA key support
#ifndef IAP_RSA_SUPPORT
   #define IAP_RSA_SUPPORT DISABLED
#elif (IAP_RSA_SUPPORT != ENABLED && IAP_RSA_SUPPORT != DISABLED)
   #error IAP_RSA_SUPPORT parameter is not valid
#endif

//ECDSA key support
#ifndef IAP_ECDSA_SUPPORT
   #define IAP_ECDSA_SUPPORT DISABLED
#elif (IAP_ECDSA_SUPPORT != ENABLED && IAP_ECDSA_SUPPORT != DISABLED)
   #error IAP_ECDSA_SUPPORT parameter is not valid
#endif

#if ((IAP_ENCRYPTION_SUPPORT == ENABLED) || ((IAP_SINGLE_BANK_SUPPORT == ENABLED) \
&& (IAP_EXT_MEM_ENCRYPTION_SUPPORT == ENABLED)))
#include "cipher/cipher_algorithms.h"
#include "cipher/aes.h"
#endif

#if (IAP_AUTHENTICATION_SUPPORT == ENABLED)
#include "mac/hmac.h"
#endif

#if (IAP_ENCRYPTION_SUPPORT == ENABLED) || (IAP_SINGLE_BANK_SUPPORT == ENABLED)
//Cipher context size
#define IAP_MAX_CIPHER_CONTEXT_SIZE sizeof(AesContext)
//Cipher IV size
#define IAP_MAX_CIPHER_IV_SIZE MAX_CIPHER_BLOCK_SIZE // 32
#endif

//Maximum check data context size
#if (IAP_AUTHENTICATION_SUPPORT == ENABLED)
#define IAP_MAX_CHECK_CONTEXT_SIZE sizeof(HmacContext)
#else
#define IAP_MAX_CHECK_CONTEXT_SIZE sizeof(Crc32Context)
#endif

//Maximum digest size
#define IAP_MAX_HASH_DIGEST_SIZE MAX_HASH_DIGEST_SIZE

//Signature size
#define IAP_MAX_CHECK_DATA_SIZE 512


//Forward declaration of IapContext structure
struct _IapContext;
#define IapContext struct _IapContext

/**
 * @brief Random data generation callback function
 **/

typedef error_t (*IapRandCallback)(uint8_t *data, size_t length);


/**
 *@brief Supported authentication algorithms
 **/

typedef enum
{
   IAP_AUTH_NONE,
   IAP_AUTH_HMAC
} IapAuthAlgo;


/**
 *@brief Supported signature algorithms
 **/

typedef enum
{
   IAP_SIGN_NONE,
   IAP_SIGN_RSA,
   IAP_SIGN_ECDSA
} IapSignAlgo;


/**
 * @brief IAP Crypto settings
 **/

typedef struct
{
#if ((IAP_ENCRYPTION_SUPPORT == ENABLED) || \
    ((IAP_SINGLE_BANK_SUPPORT == ENABLED) && \
    (IAP_EXT_MEM_ENCRYPTION_SUPPORT == ENABLED)))
   const CipherAlgo *cipherAlgo;                   ///<Image cipher algorithm
   CipherMode cipherMode;                          ///<Image cipher mode
   const char_t *cipherKey;                        ///<Image cipher key
   size_t cipherKeyLen;                            ///<Image cipher key size
#endif
#if ((IAP_INTEGRITY_SUPPORT == ENABLED) || \
    (IAP_SINGLE_BANK_SUPPORT == ENABLED))
   const HashAlgo *integrityAlgo;                  ///<Image integrity algorithm
#endif
#if (IAP_AUTHENTICATION_SUPPORT == ENABLED)
   IapAuthAlgo authAlgo;                           ///<Image authentication algorithm
   const HashAlgo *authHashAlgo;                   ///<Image authentication hash algorithm
   const char_t *authKey;                          ///<Image authentication key
   size_t authKeyLen;                              ///<Image authentication key size
#endif
#if (IAP_SIGNATURE_SUPPORT == ENABLED)
   IapSignAlgo signAlgo;                           ///<Image signature algorithm
   const HashAlgo *signHashAlgo;                   ///<Image signature hash algorithm
   const char_t *signKey;                          ///<Image signature public key (PEM format)
   size_t signKeyLen;                              ///<Image signature public key size
#endif
} IapCryptoSettings;


/**
 * @brief IAP user settings
 **/

typedef struct
{
   uint32_t appVersion;                         ///<Version of the current running application
	const FlashDriver *primaryFlashDriver;       ///<Driver of the primary (internal) flash memory driver 
	IapCryptoSettings imageInCrypto;             ///<Cryptographic settings used to manage an update image
#if (IAP_SINGLE_BANK_SUPPORT == ENABLED)
	const FlashDriver *secondaryFlashDriver;     ///<Driver of the secondary (external) flash memory driver    
	IapCryptoSettings imageOutCrypto;            ///<Cryptographic settings used to generate the update image
	uint16_t addrSlot1;                          ///<Address of the first slot in secondary memory
	size_t sizeSlot1;                            ///<Size of the first slot in secondary memory
#if (IAP_EXT_MEM_ENCRYPTION_SUPPORT == ENABLED)
	const char_t *psk;                           ///<PSK key used to encrypt the output image
	uint32_t pskSize;                            ///<Size of the PSK key used to encrypt the output image
#endif
#if IAP_FALLBACK_SUPPORT == ENABLED
    uint32_t addrSlot2;                         ///<Address of the second slot in secondary memory
    size_t sizeSlot2;                           ///<Size of the second slot in secondary memory
#endif
#endif
}IapSettings;


/**
 * @brief IAP states
 **/

typedef enum
{
   IAP_STATE_IDLE,
   IAP_STATE_RECV_APP_HEADER,
   IAP_STATE_RECV_APP_DATA,
   IAP_STATE_RECV_APP_CHECK,
   IAP_STATE_VALIDATE_APP,
   IAP_STATE_APP_REBOOT
} IapState;


/**
 * @brief Image context
 **/

typedef struct
{
   uint16_t imgIdx;                                   ///<Image index number
   struct _IapContext *iapContext;                    ///<Pointer to the parent IAP context

   uint32_t firmwareAddr;                             ///<Image firmware data write address
   size_t firmwareLength;                             ///<Image data firmware length
   uint32_t pos;                                      ///<Image current firmware data write position
   size_t written;                                    ///<Current written firmware data byte number

   uint8_t state;                                     ///<Image process state

   uint8_t buffer[128];                               ///<Image processing buffer
   uint8_t *bufferPos;                                ///<Position in image prcessing buffer
   size_t bufferLen;                                  ///<Number of byte in image processing buffer

#if (IAP_ENCRYPTION_SUPPORT == ENABLED || \
    ((IAP_SINGLE_BANK_SUPPORT == ENABLED) && \
    (IAP_EXT_MEM_ENCRYPTION_SUPPORT == ENABLED)))
   CipherContext cipherContext;                       ///<Image cipher context
   uint8_t cipherIv[IAP_MAX_CIPHER_IV_SIZE];          ///<Image ciphher IV
   size_t cipherIvLen;                                ///<Image cipher IV length
#endif
   uint8_t checkContext[IAP_MAX_CHECK_CONTEXT_SIZE];  ///<Image check context
   uint8_t checkData[IAP_MAX_CHECK_DATA_SIZE];        ///<Image check data buffer
   uint8_t *checkDataPos;                             ///<Position in image check data buffer
   size_t checkDataLen;                               ///<Current number of byte in image check data buffer
   size_t checkDataSize;                              ///<Image check data buffer size

   uint8_t digest[IAP_MAX_HASH_DIGEST_SIZE];          ///<Image digest buffer
   size_t digestLen;                                  ///<Image digest length

   IapCryptoSettings *imageCryptoSettings;            ///<Image cryptographic settings
   SlotDesc *slotInfo;                                ///<Image flash memory slot
} ImageContext;


/**
 * @brief IAP context
 **/

struct _IapContext
{
   IapSettings settings;      ///<IAP user settings

   FlashDesc primaryMem;      ///<Primary flash memory descriptor
   FlashDesc secondaryMem;    ///<Secondary (external) flash memory descriptor
	 
   ImageContext imageInput;   ///<Input image (received update image) context
   ImageContext imageOutput;  ///<Output image context
};

//CycloneBOOT IAP application related functions
void iapGetDefaultSettings(IapSettings *settings);
char_t *iapGetVersion(void);
error_t iapRegisterRandCallback(IapRandCallback callback);
error_t iapInit(IapContext *context, const IapSettings *settings);
error_t iapUpdate(IapContext *context, const void *data, size_t length);
error_t iapFinalize(IapContext *context);
error_t iapReboot(IapContext *context);

//Extern device MCU related function
extern uint32_t mcuGetVtorOffset(void);
extern void mcuSystemReset(void);
extern void mcuJumpToApplication(uint32_t address);

//Random data generation callback function
extern IapRandCallback iapRandCallback;

#endif // !_IAP_H
