/**
 * @file iap_auth.c
 * @brief CycloneBOOT IAP image authentication handling
 *
 * @section License
 *
 * Copyright (C) 2010-2022 Oryx Embedded SARL. All rights reserved.
 *
 * This file is part of CycloneBOOT Eval.
 *
 * This software is provided in source form for a short-term evaluation only. The
 * evaluation license expires 90 days after the date you first download the software.
 *
 * If you plan to use this software in a commercial product, you are required to
 * purchase a commercial license from Oryx Embedded SARL.
 *
 * After the 90-day evaluation period, you agree to either purchase a commercial
 * license or delete all copies of this software. If you wish to extend the
 * evaluation period, you must contact sales@oryx-embedded.com.
 *
 * This evaluation software is provided "as is" without warranty of any kind.
 * Technical support is available as an option during the evaluation period.
 *
 * @author Oryx Embedded SARL (www.oryx-embedded.com)
 * @version 2.0.0
 **/

//Switch to the appropriate trace level
#define TRACE_LEVEL IAP_TRACE_LEVEL

//Dependencies
#include "iap/iap_auth.h"
#include "debug.h"


/**
 * @brief Initialize authentification material for further authentification tag
 * computation on received firmware data and verification against 
 * received image authentification tag.
 * @param[in,out] context Pointer to the IAP context
 * @return Error code
 **/

error_t iapAuthInit(ImageContext *context)
{
#if IAP_AUTHENTICATION_SUPPORT == ENABLED
   error_t error;
   IapCryptoSettings *settings;

   //Check parameter validity
   if (context == NULL)
      return ERROR_INVALID_PARAMETER;

   //Point to the IAP user settings
   settings = (IapCryptoSettings*)context->imageCryptoSettings;

   //Initialize status code
   error = NO_ERROR;

   //Check user settings
   if (settings->authHashAlgo == NULL || settings->authAlgo == IAP_AUTH_NONE)
      return ERROR_INVALID_VALUE;

   //Is hmac authentification algo?
   if (settings->authAlgo == IAP_AUTH_HMAC)
   {
      //Initialize authentification hmac context
      error = hmacInit((HmacContext*)context->checkContext, settings->authHashAlgo, 
         settings->authKey, settings->authKeyLen);

      //Set digest length
      context->digestLen = ((HmacContext*)context->checkContext)->hash->digestSize;

      //Set check data (authentification tag) size
      context->checkDataSize = ((HmacContext*)context->checkContext)->hash->digestSize;
   }
   else
   {
      //Debug message
      TRACE_ERROR("Authentification algorithm not supported!\r\n");
      error = ERROR_NOT_IMPLEMENTED;
   }

   //Return status code
   return error;
#else
   //Not implemented
   return ERROR_NOT_IMPLEMENTED;
#endif
}


/**
 * @brief Update authentification tag computation on received firmware data bloc.
 * @param[in,out] context Pointer to the IAP context.
 * @param[in] data Firmware data bloc to process.
 * @param[in] length Length of the firmware data bloc to process.
 * @return Error code
 **/

error_t iapAuthUpdateTag(ImageContext *context, const uint8_t *data, size_t length)
{
#if IAP_AUTHENTICATION_SUPPORT == ENABLED
   error_t error;
   IapCryptoSettings* settings;

   //Check parameter validity
   if (context == NULL || data == NULL || length == 0)
      return ERROR_INVALID_PARAMETER;

   //Point to the IAP user settings
   settings = (IapCryptoSettings*)context->imageCryptoSettings;

   //Initialize status code
   error = NO_ERROR;

   //Check authentification hash algo
   if (settings->authAlgo == IAP_AUTH_NONE)
      return ERROR_INVALID_VALUE;
   
   //Is hmac authentification algo?
   if (settings->authAlgo == IAP_AUTH_HMAC)
   {
      //Update uahtentification hmac tag
      hmacUpdate((HmacContext*)context->checkContext, data, length);
   }
   else
   {
      //Debug message
      TRACE_ERROR("Authentification algorithm not supported!\r\n");
      error = ERROR_NOT_IMPLEMENTED;
   }

   //Return status code
   return error;
#else
   //Not implemented
   return ERROR_NOT_IMPLEMENTED;
#endif
}


/**
 * @brief Finalize authentification tag computation on received firmware data.
 * @param[in,out] context Pointer to the IAP context.
 * @return Error code
 **/

error_t iapAuthFinalizeTag(ImageContext *context)
{
#if IAP_AUTHENTICATION_SUPPORT == ENABLED
   error_t error;
   IapCryptoSettings* settings;

   //Check parameter validity
   if (context == NULL)
      return ERROR_INVALID_PARAMETER;

   //Point to the IAP user settings
	settings = (IapCryptoSettings*)context->imageCryptoSettings;

   //Initialize status code
   error = NO_ERROR;

   //Check authentification hash algo
   if (settings->authAlgo == IAP_AUTH_NONE)
      return ERROR_INVALID_VALUE;

   //Is hmac authentification algo?
   if (settings->authAlgo == IAP_AUTH_HMAC)
   {
      hmacFinal((HmacContext*)context->checkContext, context->digest);
   }
   else
   {
      //Debug message
      TRACE_ERROR("Authentification algorithm not supported!\r\n");
      error = ERROR_NOT_IMPLEMENTED;
   }

   //Return status code
   return error;
#else
   //Not implemented
   return ERROR_NOT_IMPLEMENTED;
#endif
}
